import os
import shutil
import h5py
from pathlib import Path
import numpy as np
import argparse
import pdb

# Assuming the 'default_parameters.h5' file is located directly inside the package folder
#default_parameters_path = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))),'default_parameters.h5')
default_parameters_path = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))),'parameters.h5')
# Determine the user's home configuration path
user_parameters_path = os.path.join(Path.home(), '.AIPyS', 'parameters.h5') # save the parameter file to .AIPyS folder. 

def ensure_user_parameters():
    """Ensure the user has their own copy of parameters.h5; if not, create it."""
    if not os.path.exists(user_parameters_path):
        os.makedirs(os.path.dirname(user_parameters_path), exist_ok=True)
        shutil.copy(default_parameters_path, user_parameters_path)
        print(f'Created a personal parameters file at {user_parameters_path}')


def update_auto_parameters(parameter_updates,user_parameters_path):
    '''
    this function is used for updating h5 file based on results
    e.g. intercept and slope from model build
    '''
    with h5py.File(user_parameters_path, 'r+') as hdf:
        for key, value in parameter_updates.items():
            if key in hdf:
                value = 'None' if value is None else value
                if isinstance(value, str):
                    hdf[key][()] = np.array(value, dtype=h5py.special_dtype(vlen=str))
                else:
                    hdf[key][...] = value
            else:
                # If the key does not exist in the HDF5 file, create a new dataset.
                if isinstance(value, str):
                    dt = h5py.special_dtype(vlen=str)
                    hdf.create_dataset(key, data=np.array(value, dtype=dt), dtype=dt)
                else:
                    hdf.create_dataset(key, data=value)
                print(f"'{key}' added to or updated in the HDF5 file with value: {value}")



def update_user_parameters(parameter_updates):
    """Example function to update parameters in the user's parameters.h5 file."""
    ensure_user_parameters()  # Ensure the user copy exists
    #pdb.set_trace()
    with h5py.File(user_parameters_path, 'r+') as hdf:
        for key, value in parameter_updates.items():
            if key in hdf:
                value = 'None' if value is None else value
                if isinstance(value, str):
                    hdf[key][()] = np.array(value, dtype=h5py.special_dtype(vlen=str))
                else:
                    hdf[key][...] = value
            else:
                # If the key does not exist in the HDF5 file, create a new dataset.
                if isinstance(value, str):
                    dt = h5py.special_dtype(vlen=str)
                    hdf.create_dataset(key, data=np.array(value, dtype=dt), dtype=dt)
                else:
                    hdf.create_dataset(key, data=value)
                print(f"'{key}' added to or updated in the HDF5 file with value: {value}")

def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
    else:
        return open(arg, 'r')  # return an open file handle

# def read_user_parameters():
#     """Example function to read parameters from the user's parameters.h5 file."""
#     ensure_user_parameters()  # Ensure the user copy exists
#     with h5py.File(user_parameters_path, 'r') as hdf:
#         # Read and return parameters as needed

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="This program provides a command-line interface for updating, adding, or displaying parameters stored in a .h5 file. It's designed to manage experimental parameters for bioinformatics or genetic studies,enabling users to easily modify values such as target numbers, gene counts, target plasmid ratios, and more. Whether you're adjusting the setup for a new experiment or reviewing configurations for an ongoing study, this tool simplifies parameter management without the need to manually edit files.", formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('--targetNum', type=int, help="Specifies the target number.")
    parser.add_argument('--geneNum', type=int, help="Specifies the number of genes.")
    parser.add_argument('--effectSgRNA', type=int, help="Specifies the number of effective sgRNAs.")
    parser.add_argument('--tpRatio', type=int, help="The target plasmid ratio, indicating the balance between target and control.")
    parser.add_argument('--n', type=int, help="Expected negative binomial rate (mu) for the distribution.")
    parser.add_argument('--p', type=float, help="Dispersion (alpha) of the negative binomial distribution.")
    parser.add_argument('--low', type=int, help="Specifies the lower limit.")
    parser.add_argument('--high', type=int, help="Specifies the upper limit.")
    parser.add_argument('--size', type=int, help="The total sum of the master (M) sample before generating H_0.")
    parser.add_argument('--FalseLimits', nargs=2, type=float, help="A tuple indicating the minimum and maximum viral particles per cell. e.g., --FalseLimits 0.01 0.5")
    parser.add_argument('--ObservationNum', nargs=2, type=int, help="The number of observations and their standard deviation to generate. e.g., --ObservationNum 70 3")
    
    args = parser.parse_args()
   
    # Construct the dictionary from provided CLI arguments
    # Exclude 'h5_file' from update_dict, as it's not a parameter to store but the file path
    update_dict = {k: v for k, v in vars(args).items() if k != 'h5_file' and v is not None}
    #pdb.set_trace()
    # Update or add parameters in the H5 file
    update_user_parameters(update_dict)


import re
import numpy as np
import pandas as pd
from sklearn import preprocessing
from tqdm import tqdm

class mapSgRNA:
    def __init__(self,df1,df2):
        '''
        :param df1: Q1 sample (original)
        :param df2: Q2 sample (active sort)
        '''
        self.df1 = df1
        self.df2 = df2
        self.mapping()
    
    def mapping(self):
        '''
        :param uniqueSgRNA: list of strings, unique sgRNA
        :return: mapping reads according to unique sgRNA
        '''
        sgRNAdic = {'Gene':[],'sgRNA':[],"reads_ctrl":[],"reads_activate":[]}
        uniqueSgRNA = np.unique(self.df2.sgID.values)
        progress = tqdm()
        print("preparing merging data")   
        for sgRNA in uniqueSgRNA:
            sgRNAdic['sgRNA'].append(sgRNA)
            # check for non targeting:
            if re.match("non-targeting.*", sgRNA) is None:
                sgRNAdic['Gene'].append(re.sub('_._.*','', sgRNA))
            else:
                sgRNAdic['Gene'].append("non")
            # reads ctrl
            if self.df1.loc[self.df1['sgRNA'] == sgRNA, 'count'].any():
                sgRNAdic['reads_ctrl'].append(self.df1.loc[self.df1['sgRNA'] == sgRNA, 'count'].values[0])
            else:
                sgRNAdic['reads_ctrl'].append(2)
            # reads active
            sgRNAdic['reads_activate'].append(self.df2.loc[self.df2['sgID'] == sgRNA, "Q2_Reads"].values[0])
            #print('{}'.format(sgRNA))
            progress.update()
        return sgRNAdic

    @staticmethod
    def dataFrameFinal(sgRNAdic):
        sgRNAdic['log2FoldChange'] = np.log(sgRNAdic['reads_activate']) - np.log(sgRNAdic['reads_ctrl'])
        sgRNAdic['scaleLog2FoldChange'] = preprocessing.scale(sgRNAdic['log2FoldChange'])
        df = pd.DataFrame(sgRNAdic)
        return df
    
    @staticmethod
    def Create_origVSactivDF(df_orig,df_active):
        '''
        Function takes data frame output from saveData module.One data frame is active sample and the other is output of the original (sgRNA and counts columns). Output count list acording to df_active sgRNA. 
        Example:
        df_active (Q1): 'df_0pymc_mu_100_num_4.csv'
        df_orig : 'Orig_2pymc_mu_100_num_4.csv'
        '''
        sgRNA = df_active.sgRNA.values
        count_orig = []
        for sg in sgRNA:
            if df_orig.loc[df_orig['sgRNA'] == sg, df_orig.columns[2]].any():
                count_orig.append(df_orig.loc[df_orig['sgRNA'] == sg,'count'].values[0])
            else:
                count_orig.append(2)
        return count_orig


import numpy as np
import os
import pandas as pd
import pymc as pm
import matplotlib.pyplot as plt
import seaborn as sns
import tqdm as tqdm
import requests
from io import BytesIO
import tempfile

RANDOM_SEED = 8927
rng = np.random.default_rng(RANDOM_SEED)

import random


class SimInit:
    """
    Manages initiation of simulations based on user-defined parameters.

    This class serves as the starting point for simulations, taking user-specified parameters
    and saving them into an HDF5 (.h5) file for persistent storage. It acts as a base class,
    passing the initialized parameters to inherited classes responsible for executing
    the actual simulation tasks.

    Parameters are encapsulated in an HDF5 file due to its efficiency in handling large
    datasets and its hierarchical structure, which is suitable for complex simulations.

    The class is designed to be inherited by specific simulation classes, which will
    implement the detailed logic necessary for performing the simulation based on the
    initial parameters.
    
    parameters
    ==========
    targetNum: int
    geneNum: int
    effectSgRNA: int, 
        between 0 to 5
    getData: bool,
        data load to memeory
    return
    ======
    dfSubset: table
    effective_sgRNA_flat: list
    
    """
    def __init__(self, targetNum, geneNum, effectSgRNA, getData):
        self.targetNum = targetNum
        self.geneNum = geneNum
        self.effectSgRNA = effectSgRNA
        self.getData = getData
        

    #reading files stay in memory (google colab)
    def download_data_file_and_read(self,url, data_key):
        """
        Downloads a file from the given URL to a named temporary file and reads it.
        """
        with requests.get(url, stream=True) as r:
            with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
                for chunk in r.iter_content(chunk_size=8192):
                    tmp_file.write(chunk)
                
                # The file needs to be closed before reading
                tmp_file_name = tmp_file.name

        # Now you can read from the downloaded file
        try:
            dfinit = pd.read_hdf(tmp_file_name, key=data_key)
        except Exception as e:
            raise IOError(f"Cannot read HDF5 data: {e}")
        # Optionally, if you'd like to clean up immediately after reading
        # os.unlink(tmp_file_name)
        return dfinit
    
    def loading_data(self):
        if self.geneNum is None or self.targetNum is None or self.targetNum > self.geneNum or self.targetNum==0:
            raise ValueError("Invalid or missing parameters. Please ensure targetNum is not 0, parameters are not None, and targetNum is less than or equal to geneNum.") 
        else:
            data_key = 'dataset' 
            if self.getData:
                # Define the URL of the raw data file on GitHub
                url = "https://raw.githubusercontent.com/gkanfer/AIPySdeAnalyzer/main/datasgRNA.h5"
                print(f"Downloading data file from {url}")
                dfinit = self.download_data_file_and_read(url,data_key)
            else:
                file_path = 'datasgRNA.h5'                
                dfinit = pd.read_hdf(file_path, key=data_key)
            df = dfinit.iloc[8:,:]
            unique_genes = df['gene'].unique()
            selected_genes = random.sample(list(unique_genes), self.geneNum)  # Randomly select
            dfSubset = df[df['gene'].isin(selected_genes)]
            target_genepool = random.sample(list(selected_genes), self.targetNum)
            effective_sgRNA = [dfSubset.loc[dfSubset['gene'] == gene,'sgID'].to_list()[0:self.effectSgRNA] for gene in set(target_genepool)]
            effective_sgRNA_flat = [item for sublist in effective_sgRNA for item in sublist]
            return dfSubset, effective_sgRNA_flat
        
        
def download_file(url, local_filename):
    """
    Downloads a file from the given URL and saves it locally.
    """
    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(local_filename, 'wb') as f:
            for chunk in r.iter_content(chunk_size=8192):
                f.write(chunk)


def ensure_data_file():
    """
    Ensures the required datasgRNA.h5 file is available.
    """
    data_filename = "datasgRNA.h5"
    data_path = os.path.join(os.path.expanduser("~"), data_filename)
    
    # Check if the file doesn't already exist
    if not os.path.exists(data_path):
        print(f"Downloading {data_filename}...")
        url = "https://raw.githubusercontent.com/gkanfer/AIPySdeAnalyzer/main/datasgRNA.h5"
        download_file(url, data_path)
    else:
        print(f"{data_filename} already exists.")
    
    return data_path


import numpy as np
import tqdm

class UploadData:
    def __init__(self, data):
        self.data = data
    
    def defineData(self, name):
        return {name: self.data}

class MapSimMOI:
    def __init__(self, data_list):
        self.data_dict = self.upload_data(data_list)
        self.df_map = self.mapsgRNA()
    
    def upload_data(self, data_list):
        data_dict = {}
        for name, data in data_list:
            uploader = UploadData(data)
            data_dict.update(uploader.defineData(name))
        return data_dict
    
    @staticmethod
    def unpack_sgRNA(df,colCount=1,colsgRNA=0):
        sgRNA = []
        for i in tqdm.tqdm(range(len(df))):
            count = df.loc[i,colCount]
            sgComb = df.loc[i,colsgRNA]
            for sg in sgComb.split("#"):
                sgRNA = sgRNA + [sg]*count
        return sgRNA
    
    def mapsgRNA(self):
        """_summary_
        dforig: 'sgRNA', 'count'
        dfQ2:'sgID', 'Q2_Reads'
        df_m:'sgID''count_sim'
        """
        df_map_dict_origSgRNA = {}
        sgRNA_uniqe = []
        sampleId = []
        # Unpack sgRNA from etch table
        for name, data in self.data_dict.items():
            if 'orig' in name:
                colCount='count'
                colsgRNA='sgRNA'
            elif 'M' in name:
                colCount='count_sim'
                colsgRNA='sgID'
            else:
                colCount='Q2_Reads'
                colsgRNA='sgID'
            sgRNA = self.unpack_sgRNA(data[0],colCount=colCount,colsgRNA=colsgRNA)
            df_map_dict_origSgRNA[name] = sgRNA # dictonery of list by table name
            sgRNA_uniqe = sgRNA_uniqe + sgRNA
            sampleId.append(name)
        sgRNA_uniqe = np.unique(np.array(sgRNA_uniqe))
        # Travers through the unique map and count sgRNA observed per table.
        df_map_dict = {"sgRNA":[]}
        for samp in sampleId:
            df_map_dict[samp] = [] 
        print("unpacking")
        for sg in tqdm.tqdm(sgRNA_uniqe):
            df_map_dict["sgRNA"].append(sg)
            for samp in sampleId:
                count = df_map_dict_origSgRNA[samp].count(sg)
                if count == 0:
                    count = 1
                df_map_dict[samp].append(count)
        return df_map_dict 


import os
import glob
import arviz as az
import numpy as np
import pandas as pd
from pytensor.tensor import TensorVariable
from typing import Optional, Tuple

import pymc as pm
import seaborn as sns
# import aesara
# import aesara.tensor as at
import pytensor.tensor as pt
import matplotlib.pyplot as plt
import xarray as xr
import re
from scipy import stats
from scipy.special import logit
from scipy.special import expit as logist
from matplotlib.axes import Axes
import matplotlib.colors as mcolors
import matplotlib.patches as mpatches


from tqdm import tqdm

from aipys_analyse.simulation.Simulate import Simulate
from aipys_analyse.func.unPacking import MapSimMOI

from scipy import stats
RANDOM_SEED = 8927
rng = np.random.default_rng(RANDOM_SEED)

RANDOM_SEED = 8927
np.random.seed(RANDOM_SEED)

class runSimulation(Simulate):
    def __init__(self,*args, **kwargs):
        super().__init__(*args, **kwargs)
        self.dftall = self.dataReady()
        #self.dataHandle()
    
    def dataHandle(self):
        """
        dforig: 'sgRNA', 'count'
        dfQ2:'sgID', 'Q2_Reads'
        df_m:'sgID''count_sim'
        """
        count = 1 # make sure we get 3 replicates
        container = {}
        data_list = []
        while count <= 3:
            dforig ,dfQ1,dfQ2,df_m = self.simulation()
            #df = mapSgRNA(df1 = dforig, df2 = dfQ2)
            key_orig = "df"+str(count)+"_orig"
            key_active = "df"+str(count)
            key_M = "df"+str(count)+"_M"
            container[key_orig] = dforig
            container[key_active] = dfQ2
            container[key_M] = df_m
            data_list = data_list + [(key_active,[container[key_active]]),(key_orig,[container[key_orig]]),(key_M,[container[key_M]])]
            count += 1
        return  data_list   
    
    def dataReady(self):
        data_list = self.dataHandle()
        sampleId = MapSimMOI(data_list)
        df = pd.DataFrame(sampleId.df_map)
        df['Gene'] = [re.sub('_._.*','', sg) for sg in df.sgRNA.values]
        df.loc[df.sgRNA.str.contains('non'),'Gene'] = 'non'
        df = pd.melt(df,id_vars = ['Gene','sgRNA'], value_vars=['df1_orig','df2_orig','df3_orig',
                                                     'df1', 'df2', 'df3','df1_M','df2_M','df3_M'])
        df["log"] = np.log(df["value"])
        df["condition"] = 1
        df.loc[df.variable.str.contains('orig'),"condition"] = 0
        df.loc[df.variable.str.contains('M'),"condition"] = 2
        return df
            



import numpy as np
import os
import pandas as pd
import pymc as pm
import matplotlib.pyplot as plt
import seaborn as sns
import tqdm as tqdm


RANDOM_SEED = 8927
rng = np.random.default_rng(RANDOM_SEED)

import random
from aipys_analyse.SimInit import SimInit


class Simulate(SimInit):
    """
    Initialize the simulation module inheriting the starting dataset with preset target sgRNAs.
    During the simulation, a master plasmid table (df.M) is generated using a negative binomial 
    distribution. The H_0 sample (representing conditions before screening) is generated from 
    a multinomial distribution with known confounding parameters. H_1 (post-screen sample) 
    is generated from H_1 by selecting target hits with some residual noise introduced by the user.

    Parameters
    ----------
    tpRatio : int
        The target plasmid ratio, indicating the balance between target and control.
    n : int
        Expected negative binomial rate (mu) for the distribution.
    p : float
        Dispersion (alpha) of the negative binomial distribution.
    size : int
        The total sum of the master (M) sample before generating H_0.
    FalseLimits : tuple of float
        A tuple indicating the minimum and maximum viral particles per cell. e.g. (0.01,0.5)
    ObservationNum : int
        The number of observations to generate. fov of vie e.g. 70 cells per fov with sd of 3
    *args
        Variable length argument list for parent class initialization.
    **kwargs
        Arbitrary keyword arguments for parent class initialization.

    Returns
    -------
    dfSim : DataFrame
        Simulated DataFrame, representing H_0 sample before selection.
    df_M : DataFrame
        Master plasmid DataFrame, generated from a negative binomial distribution.
    Original : DataFrame
        Original H_0 DataFrame before the screening process.
    dfQ1 : DataFrame
        DataFrame of all the non-selected targets.
    dfQ2 : DataFrame
        Target DataFrame, indicating selected hits post-screening.
    """
    def __init__(self, tpRatio, n, p,low,high, size,FalseLimits, ObservationNum, *args, **kwargs):
        self.tpRatio = tpRatio
        self.n = n #mu
        self.p = p #alpha
        self.low = low,
        self.high =high, 
        self.size = size
        self.FalseLimits = FalseLimits
        self.ObservationNum = ObservationNum
        super().__init__(*args, **kwargs)
        self.dfSubset,self.effective_sgRNA_flat = self.loading_data()
        self.dfSim,self.df_m = self.observePerRaw()
        
    def observePerRaw(self):
        ''':param
            50_000 observation are compitable with the meory usage
            seed_num intiger from 0 to 2
        '''
        df = self.dfSubset
        df['count_sim'] = pm.draw(pm.NegativeBinomial.dist(mu=self.p, alpha=self.n),draws=len(self.dfSubset), random_seed=RANDOM_SEED)
        df['count_sim_p'] = df['count_sim'].values/np.sum(df['count_sim'].values)
        df_m = df.copy()
        np.random.seed(123124)
        sgRNA = df['sgID'].values
        MOI = np.random.randint(self.low, self.high, self.size)  # control MOI levels
        # MOI metrix
        seed_random = random.randint(10000, 99999) # seed generated from random module
        MOI_mat = pm.draw(pm.Multinomial.dist(n = MOI,p = df['count_sim_p'].values),draws=1, random_seed=seed_random)
        # Assign 1 the hits above 1
        MOI_mat = np.where(MOI_mat > 1, 1, MOI_mat)
        # Replace ones with corresponding sgRNA from map and assign activity
        Qoriginal = {}
        progress = tqdm.tqdm()
        print("generates ht0 sample")
        for i,row in enumerate(MOI_mat):
            temp_sgRNA = ['#'.join(filter(None, (sgRNA[col] if val==1 else '' for col, val in enumerate(row))))]
            activity = [any(word in sg for word in self.effective_sgRNA_flat) for sg in temp_sgRNA]
            Qoriginal[i] = temp_sgRNA + activity
            progress.update()
        df = pd.DataFrame.from_dict(Qoriginal, orient='index', columns=['sgID', 'Active'])
        df.reset_index(level=0, inplace=True)
        df.rename(columns = {'index':'cell'}, inplace = True)
        table = df.sample(frac=1, replace=False, random_state=1564743454, axis=0, ignore_index=True)
        return table,df_m

    def simulation(self):
        '''
        :param FalseLimits, tuple, precantage list of False Positive
        :param ObservationNum, tuple, mean and standard deviation
        '''
        Original = self.dfSim
        dfQ1 = {}
        dfQ2 = {}
        fpRate = [arr for arr in np.arange(self.FalseLimits[0], self.FalseLimits[1], self.FalseLimits[0])]
        progress = tqdm.tqdm()
        while True:
            progress.update()
            FOV = int(np.random.normal(self.ObservationNum[0],self.ObservationNum[1]))
            if FOV > len(self.dfSim):
                break
            dfTemp = self.dfSim.iloc[:FOV,:]
            # shorten the table by fov
            self.dfSim = self.dfSim.iloc[FOV+1:,:]
            idxTruePostive = dfTemp.index[dfTemp['Active']].tolist()
            if len(idxTruePostive) > 0:
                TruePositiveSGs = dfTemp.loc[idxTruePostive, 'sgID'].to_list()
                dfTemp = dfTemp.drop(idxTruePostive)
                for sg in TruePositiveSGs:
                    if sg in dfQ2.keys():
                        dfQ2[sg] += 1
                    else:
                        dfQ2[sg] = 1
            selFP = int(len(dfTemp.index.to_list()) * random.sample(fpRate,1)[0])
            if selFP > 0:
                TruePositiveSGs =  dfTemp['sgID'].sample(n = selFP).to_list()
                for sg in TruePositiveSGs:
                    dfTemp = dfTemp[dfTemp.sgID != sg]
                    if sg in dfQ2.keys():
                        dfQ2[sg] += 1
                    else:
                        dfQ2[sg] = 1
            sgRNAexclude =  dfTemp['sgID'].to_list()
            for sg in sgRNAexclude:
                if sg in dfQ1.keys():
                    dfQ1[sg] += 1
                else:
                    dfQ1[sg] = 1
        df_m = self.df_m.reset_index()
        df_Q1 = pd.DataFrame({"sgID": list(dfQ1.keys()), "Q1_Reads": list(dfQ1.values())}).reset_index()
        df_Q2 = pd.DataFrame({"sgID": list(dfQ2.keys()), "Q2_Reads": list(dfQ2.values())}).reset_index()
        origDict = Original['sgID'].value_counts().to_dict()
        df_orig = pd.DataFrame({'sgRNA':[sgr for sgr in origDict.keys()],'count':[count for count in origDict.values()]}).reset_index()
        print("Data generation is done")
        return df_orig,df_Q1,df_Q2,df_m


import h5py
import numpy as np
import os
import shutil
from pathlib import Path
import argparse

params = {
    "targetNum": 5,
    "geneNum": 100,
    "effectSgRNA": 4,
    "tpRatio": 40,
    "n": 10,
    "p": 0.1,
    "low": 1,
    "high": 5,
    "size": 1_000,
    "FalseLimits": (0.01,0.5),
    "ObservationNum": (10,3)
    }

# Specify the HDF5 file name
filename = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))),'parameters.h5')
user_parameters_path = os.path.join(Path.home(), '.AIPyS', 'parameters.h5')

def parametersGnert():
    # Open a new HDF5 file
    with h5py.File(filename, 'w') as hdf:
        # Iterate through dictionary items and save them to the HDF5 file
        for key, value in params.items():
            # Convert None values to a recognizable string, as NoneType can't be directly stored
            if value is None:
                value = 'None'
            # Check if value needs to be stored as a string
            if isinstance(value, str):
                dt = h5py.special_dtype(vlen=str)  # special dtype for variable-length strings
                dset = hdf.create_dataset(key, (1,), dtype=dt)
                dset[0] = value
            else:
                # Directly store if the value is numeric
                hdf.create_dataset(key, data=value)
    print(f"Parameters saved to {filename}")

def resetParameters():
    if not os.path.exists(user_parameters_path):
        #if missing creat
        os.makedirs(os.path.dirname(user_parameters_path), exist_ok=True)
        shutil.copy(filename, user_parameters_path)
        print(f'Created a personal parameters file at {user_parameters_path}')
    else:
        #reset
        parametersGnert()
        shutil.copy(filename, os.path.join(Path.home(), '.AIPyS'))
        print(f'personal parameters reset at file at {user_parameters_path}')

def display_parameters():
    """Read and display parameters from the user's parameters.h5 file."""
    with h5py.File(user_parameters_path, 'r') as hdf:
        print("Current parameters stored in the H5 file:")
        for key in hdf.keys():
            print(f"{key}: {hdf[key][()]}")

def main():
    parser = argparse.ArgumentParser(description='Manage parameters file', formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('action', choices=['generate', 'reset', 'display'],
                        help="generate: Create a new parameters file with default values."
                             "reset: Reset the parameters file to default values."
                             "display: Show current values in the parameters file.")
    args = parser.parse_args()
    
    if args.action == 'generate':
        parametersGnert()
    elif args.action == 'reset':
        resetParameters()
    elif args.action == 'display':
        display_parameters()
    else:
        raise ValueError("Unknown action. Available actions: generate, reset, display")

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages,find_namespace_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name="AIPySdeAnalyzer",
    version="0.2.1",
    install_requires=required,
    packages=find_packages(include=['aipys_analyse','aipys_analyse.simulation','aipys_analyse.CLI','aipys_analyse.func']),
    entry_points={
        'console_scripts': [
        'updateParameters=aipys_analyse.CLI.setParameters:update_user_parameters',
        'load-parameters=aipys_analyse.CLI.loadParametres:main', 
            ],
    },
    author="Gil Kanfer",
    author_email="gil.kanfer.il@gmail.com",
    description="AI Powered Photoswitchable Screen analysis",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="",  # Consider adding a URL if you have a project page or source repository
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',  # Specify the minimum version of Python required
)